var Request = require("./core/Request").Request;
var NoValidTargetDefined = require("./errors/NoValidTargetDefined").NoValidTargetDefined;

class Roux {
    constructor(options) {

        this.routeOptions = {
            cache: !!options.cache,
            as: options.as ? options.as : "",
            namespace: options.namespace ? options.namespace : ""
        };

        this.baseNamespace = "App\\Http\\Controllers";

        this.contentMimeTypes = {
            urlencoded: "application/x-www-form-urlencoded",
            text: "text/plain",
            form: "multipart/form-data",
            xml: "application/xml",
            json: "application/json"
        }
    }

    setup(options) {
        this.routeOptions = Object.assign(this.routeOptions, options)
    }

    getDataType(type) {
        return this.contentMimeTypes.hasOwnProperty(type)
            ? this.contentMimeTypes[type]
            : type;
    }

    getRouteAction(action) {
        var namespace = this.baseNamespace +
            (this.routeOptions.namespace
                ? "\\" + this.routeOptions.namespace + "\\"
                : "");
        return namespace + action
    }

    getRouteName(name) {
        return this.routeOptions.as + name;
    }

    organiseOptions(options) {
        const optionData = {};

        if (options.hasOwnProperty("credentials")) {
            optionData["credentials"] = options.credentials;
        }
        if (options.hasOwnProperty("referrer")) {
            optionData["referrer"] = options.referrer;
        }
        if (options.hasOwnProperty("redirect")) {
            optionData["redirect"] = options.redirect;
        }
        if (options.hasOwnProperty("cache")) {
            optionData["cache"] = options.cache;
        }
        if (options.hasOwnProperty("mode")) {
            optionData["mode"] = options.mode;
        }
        if (options.hasOwnProperty("headers")) {
            optionData["headers"] = options.headers;
            optionData.headers["X-Requested-With"] = "XMLHttpRequest";
        } else {
            optionData["headers"] = {
                "X-Requested-With": "XMLHttpRequest"
            };
        }
        if (options.hasOwnProperty("data")) {
            optionData["body"] = options.data;
        }
        if (options.hasOwnProperty("method")) {
            optionData["method"] = options.method;
        }
        if (options.hasOwnProperty("csrf")) {
            optionData.headers["X-CSRF-TOKEN"] = options.csrf;
        }
        if (options.hasOwnProperty("contentType")) {
            optionData.headers["Content-Type"] = this.getDataType(options.contentType)
        }
        return optionData
    };

    wrapParams(params) {
        return Array.isArray(params)
            ? params
            : [params]
    };

    call(name, params = [], options = {}) {
        let data;

        if (typeof name == "object" && name instanceof Object) {
            data = {
                params: this.wrapParams(name.params),
                name: this.getRouteName(name.route),
                action: this.getRouteAction(name.action),
                options: this.organiseOptions(name)
            }
        } else {
            data = {
                params: this.wrapParams(params),
                options: this.organiseOptions(options)
            };

            if (name.indexOf(".") !== -1)
                data["name"] = this.getRouteName(name);
            else if (name.indexOf("@") !== -1)
                data["action"] = this.getRouteAction(name);
            else throw new NoValidTargetDefined(name)
        }
        const request = new Request(data, this.routeOptions);

        return request.send();
    };
}

module.exports = Roux;

